@@ -136,7 +136,7 @@ SuperLearner <- function(Y, X, newX = NULL, family = gaussian(), SL.library,
 			} else {
 				out[, s] <- testAlg$pred
 			}
-			if(verbose) message(paste("CV", libraryNames[s]))
+			if (verbose) message(paste("CV", libraryNames[s]))
 		} #end library
 	  invisible(out)
 	}
@@ -144,28 +144,32 @@ SuperLearner <- function(Y, X, newX = NULL, family = gaussian(), SL.library,
   # additional steps to put things in the correct order
   # rbind unlists the output from lapply
   # need to unlist folds to put the rows back in the correct order
-	time_train = system.time({
-	  Z[unlist(validRows, use.names = FALSE), ] <- do.call('rbind', lapply(validRows, FUN = .crossValFUN, Y = Y, dataX = X, id = id, obsWeights = obsWeights, library = library, kScreen = kScreen, k = k, p = p, libraryNames = libraryNames))
+	time_train_start = proc.time()
 
-    # Check for errors. If any algorithms had errors, replace entire column with
-    # 0 even if error is only in one fold.
-    errorsInCVLibrary <- apply(Z, 2, function(x) any(is.na(x)))
-    if(sum(errorsInCVLibrary) > 0) {
-		  Z[, as.logical(errorsInCVLibrary)] <- 0
-	  }
-	  if(all(Z == 0)) {
-		  stop("All algorithms dropped from library")
-	  }
+	Z[unlist(validRows, use.names = FALSE), ] <- do.call('rbind', lapply(validRows, FUN = .crossValFUN, Y = Y, dataX = X, id = id, obsWeights = obsWeights, library = library, kScreen = kScreen, k = k, p = p, libraryNames = libraryNames))
 
-    # Compute weights for each algorithm in library.
-    getCoef <- method$computeCoef(Z = Z, Y = Y, libraryNames = libraryNames, obsWeights = obsWeights, control = control, verbose = verbose)
-    coef <- getCoef$coef
-    names(coef) <- libraryNames
+  # Check for errors. If any algorithms had errors, replace entire column with
+  # 0 even if error is only in one fold.
+  errorsInCVLibrary <- apply(Z, 2, function(x) anyNA(x))
+  if (sum(errorsInCVLibrary) > 0) {
+	  Z[, as.logical(errorsInCVLibrary)] <- 0
+	}
+	if (all(Z == 0)) {
+	 stop("All algorithms dropped from library")
+	}
 
-	}) # Complete timing.
+  # Compute weights for each algorithm in library.
+  getCoef <- method$computeCoef(Z = Z, Y = Y, libraryNames = libraryNames,
+                                obsWeights = obsWeights, control = control,
+                                verbose = verbose,
+                                errorsInLibrary = errorsInCVLibrary)
+  coef <- getCoef$coef
+  names(coef) <- libraryNames
+
+  time_train = proc.time() - time_train_start
 
   # Set a default in case the method does not return the optimizer result.
-  if(!("optimizer" %in% names(getCoef))) {
+  if (!("optimizer" %in% names(getCoef))) {
     getCoef["optimizer"] <- NA
   }
 
@@ -186,69 +190,83 @@ SuperLearner <- function(Y, X, newX = NULL, family = gaussian(), SL.library,
     return(out)
   }
 
-  time_predict = system.time({
+  time_predict_start = proc.time()
 
-    whichScreen <- t(sapply(library$screenAlgorithm, FUN = .screenFun, list = list(Y = Y, X = X, family = family, id = id, obsWeights = obsWeights)))
+  whichScreen <- t(sapply(library$screenAlgorithm, FUN = .screenFun, list = list(Y = Y, X = X, family = family, id = id, obsWeights = obsWeights)))
 
-    # change to sapply?
-    # for(s in 1:k) {
-    #   testAlg <- try(do.call(library$library$predAlgorithm[s], list(Y = Y, X = subset(X, select = whichScreen[library$library$rowScreen[s], ], drop=FALSE), newX = subset(newX, select = whichScreen[library$library$rowScreen[s], ], drop=FALSE), family = family, id = id, obsWeights = obsWeights)))
-    #   if(inherits(testAlg, "try-error")) {
-    #     warning(paste("Error in algorithm", library$library$predAlgorithm[s], " on full data", "\n  The Algorithm will be removed from the Super Learner (i.e. given weight 0) \n" ))
-    #     errorsInLibrary[s] <- 1
-    #   } else {
-    #     predY[, s] <- testAlg$pred
-    #   }
-    #   if(control$saveFitLibrary) {
-    #     fitLibrary[[s]] <- testAlg$fit
-    #   }
-    #   if(verbose) {
-    #     message(paste("full", libraryNames[s]))
-    #   }
-    # }
-    .predFun <- function(index, lib, Y, dataX, newX, whichScreen, family, id, obsWeights, verbose, control, libraryNames) {
-      pred_fn = get(lib$predAlgorithm[index], envir = env)
-      testAlg <- try(do.call(pred_fn, list(Y = Y, X = subset(dataX, select = whichScreen[lib$rowScreen[index], ], drop=FALSE), newX = subset(newX, select = whichScreen[lib$rowScreen[index], ], drop=FALSE), family = family, id = id, obsWeights = obsWeights)))
-      # testAlg <- try(do.call(lib$predAlgorithm[index], list(Y = Y, X = dataX[, whichScreen[lib$rowScreen[index], drop = FALSE]], newX = newX[, whichScreen[lib$rowScreen[index], drop = FALSE]], family = family, id = id, obsWeights = obsWeights)))
-      if (inherits(testAlg, "try-error")) {
-        warning(paste("Error in algorithm", lib$predAlgorithm[index], " on full data", "\n  The Algorithm will be removed from the Super Learner (i.e. given weight 0) \n" ))
-        out <- rep.int(NA, times = nrow(newX))
-      } else {
-        out <- testAlg$pred
-        if (control$saveFitLibrary) {
-          eval(bquote(fitLibrary[[.(index)]] <- .(testAlg$fit)), envir = fitLibEnv)
-        }
+  # change to sapply?
+  # for(s in 1:k) {
+  #   testAlg <- try(do.call(library$library$predAlgorithm[s], list(Y = Y, X = subset(X, select = whichScreen[library$library$rowScreen[s], ], drop=FALSE), newX = subset(newX, select = whichScreen[library$library$rowScreen[s], ], drop=FALSE), family = family, id = id, obsWeights = obsWeights)))
+  #   if(inherits(testAlg, "try-error")) {
+  #     warning(paste("Error in algorithm", library$library$predAlgorithm[s], " on full data", "\n  The Algorithm will be removed from the Super Learner (i.e. given weight 0) \n" ))
+  #     errorsInLibrary[s] <- 1
+  #   } else {
+  #     predY[, s] <- testAlg$pred
+  #   }
+  #   if(control$saveFitLibrary) {
+  #     fitLibrary[[s]] <- testAlg$fit
+  #   }
+  #   if(verbose) {
+  #     message(paste("full", libraryNames[s]))
+  #   }
+  # }
+  .predFun <- function(index, lib, Y, dataX, newX, whichScreen, family, id, obsWeights, verbose, control, libraryNames) {
+    pred_fn = get(lib$predAlgorithm[index], envir = env)
+    testAlg <- try(do.call(pred_fn, list(Y = Y,
+                                         X = subset(dataX,
+                                                    select = whichScreen[lib$rowScreen[index], ], drop=FALSE),
+                                         newX = subset(newX, select = whichScreen[lib$rowScreen[index], ], drop=FALSE),
+                                         family = family, id = id, obsWeights = obsWeights)))
+    # testAlg <- try(do.call(lib$predAlgorithm[index], list(Y = Y, X = dataX[, whichScreen[lib$rowScreen[index], drop = FALSE]], newX = newX[, whichScreen[lib$rowScreen[index], drop = FALSE]], family = family, id = id, obsWeights = obsWeights)))
+    if (inherits(testAlg, "try-error")) {
+      warning(paste("Error in algorithm", lib$predAlgorithm[index], " on full data", "\n  The Algorithm will be removed from the Super Learner (i.e. given weight 0) \n" ))
+      out <- rep.int(NA, times = nrow(newX))
+    } else {
+      out <- testAlg$pred
+      if (control$saveFitLibrary) {
+        eval(bquote(fitLibrary[[.(index)]] <- .(testAlg$fit)), envir = fitLibEnv)
       }
-      if (verbose) {
-        message(paste("full", libraryNames[index]))
-      }
-      invisible(out)
     }
+    if (verbose) {
+      message(paste("full", libraryNames[index]))
+    }
+    invisible(out)
+  }
 
 
-    predY <- do.call('cbind', lapply(seq(k), FUN = .predFun, lib = library$library, Y = Y, dataX = X, newX = newX, whichScreen = whichScreen, family = family, id = id, obsWeights = obsWeights, verbose = verbose, control = control, libraryNames = libraryNames))
+  predY <- do.call('cbind', lapply(seq(k), FUN = .predFun,
+                                   lib = library$library, Y = Y, dataX = X,
+                                   newX = newX, whichScreen = whichScreen,
+                                   family = family, id = id,
+                                   obsWeights = obsWeights, verbose = verbose,
+                                   control = control,
+                                   libraryNames = libraryNames))
 
-    # check for errors
-	  errorsInLibrary <- apply(predY, 2, function(xx) any(is.na(xx)))
-	  if(sum(errorsInLibrary) > 0) {
-		  if(sum(coef[as.logical(errorsInLibrary)]) > 0) {
-			  warning(paste("re-running estimation of coefficients removing failed algorithm(s) \n Orignial coefficients are: \n", coef, "\n"))
-			  Z[, as.logical(errorsInLibrary)] <- 0
-			  if(all(Z == 0)) {
-				  stop("All algorithms dropped from library")
-			  }
-        getCoef <- method$computeCoef(Z = Z, Y = Y, libraryNames = libraryNames, obsWeights = obsWeights, control = control, verbose = verbose)
-        coef <- getCoef$coef
-        names(coef) <- libraryNames
-		  } else {
-			  warning("coefficients already 0 for all failed algorithm(s)")
+  # check for errors
+  errorsInLibrary <- apply(predY, 2, function(algorithm) anyNA(algorithm))
+  if (sum(errorsInLibrary) > 0) {
+	  if (sum(coef[as.logical(errorsInLibrary)]) > 0) {
+		  warning(paste0("Re-running estimation of coefficients removing failed algorithm(s)\n",
+		                 "Original coefficients are: \n", paste(coef, collapse = ", "), "\n"))
+		  Z[, as.logical(errorsInLibrary)] <- 0
+		  if (all(Z == 0)) {
+			  stop("All algorithms dropped from library")
 		  }
+      getCoef <- method$computeCoef(Z = Z, Y = Y, libraryNames = libraryNames,
+                                    obsWeights = obsWeights, control = control,
+                                    verbose = verbose,
+                                    errorsInLibrary = errorsInLibrary)
+      coef <- getCoef$coef
+      names(coef) <- libraryNames
+	  } else {
+		  warning("Coefficients already 0 for all failed algorithm(s)")
 	  }
+  }
 
-    # Compute super learner predictions on newX.
-	  getPred <- method$computePred(predY = predY, coef = coef, control = control)
+  # Compute super learner predictions on newX.
+  getPred <- method$computePred(predY = predY, coef = coef, control = control)
 
-  }) # Finish timing.
+	time_predict = proc.time() - time_predict_start
 
 	# Add names of algorithms to the predictions.
 	colnames(predY) <- libraryNames
